//////////////////////////////////////////////////////////////////////
// SimpleSa.cpp: implementation for the Simple Sa class
//////////////////////////////////////////////////////////////////////
#include "SimpleSa.h"

namespace moglclass
{

	SimpleSa::SimpleSa()
	{
		init();
	}

	SimpleSa::~SimpleSa()
	{
	}

	void SimpleSa::init()
	{
		pipe[0].set(glm::vec3(0, 0, 0), glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.7f, 1.0f, 0.0f));
		pipe[1].set(glm::vec3(1.4f, 0.0f, 0.0f), glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(-0.7f, 1.0f, 0.0f));
		pipe[2].set(glm::vec3(2.0f, 0.0f, 0.0f), glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.0f, 1.0f, 0.0f));
		pipe[3].set(glm::vec3(2.3f, 0.35f, 0.0f), glm::vec3(0.5f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.0f, 0.3f, 0.0f));

		// keyframe animation
		std::vector<KeyFrame> keyframes;
		keyframes.push_back(KeyFrame(glm::vec3(2, -5, 0), 0));
		keyframes.push_back(KeyFrame(glm::vec3(-1, -3, 0), 2000));
		keyframes.push_back(KeyFrame(glm::vec3(1, -1, 0), 4000));
		keyframes.push_back(KeyFrame(glm::vec3(0, 0, 0), 6000));
		keyframes.push_back(KeyFrame(glm::vec3(1, 1, 0), 8000));
		keyframes.push_back(KeyFrame(glm::vec3(-1, 3, 0), 10000));
		keyframes.push_back(KeyFrame(glm::vec3(2, 5, 0), 12000));
		setKeyframeAnimation(keyframes);
	}

	void SimpleSa::draw(Program* p, glm::mat4& projection, glm::mat4& view, glm::mat4& model)
	{
		p->useProgram();
		p->setUniform("gProjection", projection);
		p->setUniform("gView", view);

		glm::mat4 mMatrix = model * bodyTransform;
		p->setUniform("gModel", mMatrix);
		for (int i = 0; i < 4; i++)	pipe[i].draw();
	}

	bool SimpleSa::update(float elapsedTime)
	{
		if (active)
		{
			if (curve)
			{
				curve->updatePosition(elapsedTime);
				position = curve->getPosition();
				bodyTransform = glm::translate(glm::mat4(1.0f), position); // RHS x+ right

				// if time is greater than duration, then set time to duration and set active to false
				if (!curve->getLoop() && elapsedTime >= curve->getDuration())
				{
					done = true;
					active = false;
				}
			}
			init();
		}
		return true;
	}

	void SimpleSa::setKeyframeAnimation(std::vector<KeyFrame> frames_)
	{
		curve = new CatmullRomCurveAnimation(frames_, true);
	}

}